

#ifndef _SPEC_H_
#define _SPEC_H

#include "esp_wifi.h"
#include "nvs_flash.h"
#include "esp_system.h"
#include "esp_log.h"

#include "lwip/sockets.h"
#include "lwip/dns.h"
#include "lwip/netdb.h"
#include "lwip/err.h"

#define EASY

#ifdef EASY //EASY for BLE Scanner input as text
  #define WIFI_SSID      'A'//0x01
  #define WIFI_PASS      'B'//0x02
  #define WIFI_CONN      'C'//0x03
  #define BLE_WIFI_STATUS    'D'//0x04
#else
  #define WIFI_SSID      0x01
  #define WIFI_PASS      0x02
  #define WIFI_CONN      0x03
  #define BLE_WIFI_STATUS    0x04
#endif

#define PART_1  0x10
#define PART_2  0x20
#define PART_3  0x30
#define PART_4  0x40
#define PART_5  0x50

#define WIFI_STATUS_CONN                  0x01     //connected
#define WIFI_STATUS_NCONN                 0x02     //not connected
// #define WIFI_STATUS_CONN_COMP_SUCCESS     0x03     //connection process complete with success
// #define WIFI_STATUS_CONN_COMP_FAIL        0x04     //connection process complete with failure
#define WIFI_STATUS_REV_SSID              0x05     //status to ACK receive packet
#define WIFI_STATUS_REV_PASS              0x06

#define BUFFER_MAX_SIZE   64

#define SPEC_TAG "SPEC"

static EventGroupHandle_t wifi_event_group;
const static int CONNECTED_BIT = BIT0;


typedef struct {
  bool available;
  char buffer[BUFFER_MAX_SIZE];
  int length;
}buffer_t;

buffer_t receive_buffer;
buffer_t transmit_buffer;

char CONFIG_WIFI_SSID[32];
int CONFIG_WIFI_SSID_LEN;
char CONFIG_WIFI_PASSWORD[64];
int CONFIG_WIFI_PASSWORD_LEN;

int BLE_WIFI_COMM_STATUS = WIFI_STATUS_NCONN;
bool BLE_WIFI_COMM_STATUS_CHANGED = false;

wifi_config_t wifi_config_BLE;

void update_ble_wifi_transmit()
{
  transmit_buffer.buffer[0]=BLE_WIFI_STATUS;
  transmit_buffer.buffer[1]=BLE_WIFI_COMM_STATUS;
  transmit_buffer.length=2;
  transmit_buffer.available = true;
}
void update_ble_wifi_comm_status(uint8_t state)
{
  BLE_WIFI_COMM_STATUS = state;
  BLE_WIFI_COMM_STATUS_CHANGED = true;
  update_ble_wifi_transmit();
}

esp_err_t wifi_event_handler(void *ctx, system_event_t *event)
{
    switch(event->event_id) {
        case SYSTEM_EVENT_STA_START:
          ESP_LOGI(SPEC_TAG,"SYSTEM_EVENT_STA_START ");
          	ESP_ERROR_CHECK(esp_wifi_connect());
            break;

        case SYSTEM_EVENT_STA_GOT_IP:
          xEventGroupSetBits(wifi_event_group, CONNECTED_BIT);
            //init app here
            ESP_LOGI(SPEC_TAG,"WIFI got ip.");
          update_ble_wifi_comm_status(WIFI_STATUS_CONN);
            break;
        case SYSTEM_EVENT_STA_DISCONNECTED:
            /* This is a workaround as ESP32 WiFi libs don't currently
               auto-reassociate. */
            xEventGroupClearBits(wifi_event_group, CONNECTED_BIT);
            ESP_LOGI(SPEC_TAG,"WIFI DISCONNECTED.");
            update_ble_wifi_comm_status(WIFI_STATUS_NCONN);
            	ESP_ERROR_CHECK(esp_wifi_connect());
            break;
        case SYSTEM_EVENT_AP_START:
        ESP_LOGI(SPEC_TAG,"SYSTEM_EVENT_AP_START ");
            break;
        case SYSTEM_EVENT_AP_STOP:
            ESP_LOGI(SPEC_TAG,"SYSTEM_EVENT_AP_STOP ");
            update_ble_wifi_comm_status( WIFI_STATUS_NCONN );
            break;
        case  SYSTEM_EVENT_AP_STACONNECTED:
            update_ble_wifi_comm_status( WIFI_STATUS_CONN);
            ESP_LOGI(SPEC_TAG,"SYSTEM_EVENT_AP_STACONNECTED ");
            break;
        case SYSTEM_EVENT_AP_STADISCONNECTED:
            update_ble_wifi_comm_status(WIFI_STATUS_NCONN);
        ESP_LOGI(SPEC_TAG,"SYSTEM_EVENT_AP_STADISCONNECTED");
            break;
        default:
            break;
    }
    return ESP_OK;
}

void initWifi()
{
  tcpip_adapter_init();
      wifi_event_group = xEventGroupCreate();
  ESP_ERROR_CHECK(esp_event_loop_init(wifi_event_handler, NULL));
  wifi_init_config_t cfg = WIFI_INIT_CONFIG_DEFAULT();
  ESP_ERROR_CHECK(esp_wifi_init(&cfg));
  ESP_ERROR_CHECK(esp_wifi_set_storage(WIFI_STORAGE_RAM));

 // #define WIFI_CONNFING "UNICORN"
 // #define WIFI_PASSword "*******"
// wifi_config_t  wifi_config_BLE_n =  {
//          .sta = {
//              .ssid = WIFI_CONNFING,
//              .password = WIFI_PASSword,
//          },
//      };

  ESP_ERROR_CHECK(esp_wifi_set_mode(WIFI_MODE_STA));
  ESP_ERROR_CHECK(esp_wifi_set_config(ESP_IF_WIFI_STA, &wifi_config_BLE));
  ESP_LOGI(SPEC_TAG, "start the WIFI SSID:[%s] password:[%s] --1 ", wifi_config_BLE.sta.ssid, wifi_config_BLE.sta.password);//"******");
  ESP_ERROR_CHECK(esp_wifi_start());
}

void acceptor(buffer_t buf)
{
  uint8_t cmd=buf.buffer[0];
  char data[30];
  memcpy(data,&buf.buffer[1],buf.length);
  data[buf.length]=0; //null termination

  ESP_LOGI(SPEC_TAG,"data : %s",data);
#ifdef EASY
  switch(cmd)//(0x0F & cmd)) // extract the mode
#else
  switch((0x0F & cmd))
#endif
  {
    case WIFI_CONN:
    ESP_LOGI(SPEC_TAG,"WIFI connection initiate.");
    initWifi();
//    esp_wifi_connect();
    break;
    case WIFI_PASS:

        CONFIG_WIFI_PASSWORD_LEN = buf.length;
        for(int i=0;i < CONFIG_WIFI_PASSWORD_LEN ;i++)
          {
            CONFIG_WIFI_PASSWORD[i]=data[i];
          }
        CONFIG_WIFI_PASSWORD[CONFIG_WIFI_PASSWORD_LEN]=0;

        for(int i=0;i < CONFIG_WIFI_PASSWORD_LEN ;i++)//CONFIG_WIFI_PASSWORD_LEN
            {
              wifi_config_BLE.sta.password[i]=CONFIG_WIFI_PASSWORD[i];
      //        ESP_LOGI(SPEC_TAG, " WIFI SSID:[%c] password:[%c] --5 \n", CONFIG_WIFI_PASSWORD[i], wifi_config_BLE.sta.password[i]);
            }
        ESP_LOGI(SPEC_TAG,"WIFI pass: %s\n",CONFIG_WIFI_PASSWORD);

    break;
    case WIFI_SSID:   //extract
        CONFIG_WIFI_SSID_LEN = buf.length;
        for(int i=0;i < CONFIG_WIFI_SSID_LEN ;i++)
          {
            CONFIG_WIFI_SSID[i]=data[i];
            ESP_LOGI(SPEC_TAG,"CONFIG_WIFI_SSID[%d] = %c\n",i,CONFIG_WIFI_SSID[i]);
          }
          CONFIG_WIFI_SSID[CONFIG_WIFI_SSID_LEN]=0;

          for(int i=0;i < CONFIG_WIFI_SSID_LEN ;i++)//CONFIG_WIFI_SSID_LEN-1
               {
                 wifi_config_BLE.sta.ssid[i]=CONFIG_WIFI_SSID[i];
        //        ESP_LOGI(SPEC_TAG, " WIFI SSID:[%c] password:[%c] --4 \n", CONFIG_WIFI_SSID[i], wifi_config_BLE.sta.ssid[i]);
               }

      ESP_LOGI(SPEC_TAG,"WIFI ssid: %s\n",CONFIG_WIFI_SSID);

    break;
    case BLE_WIFI_STATUS: //reply back status of connection
    ESP_LOGI(SPEC_TAG,"WIFI status:");
    update_ble_wifi_comm_status(BLE_WIFI_COMM_STATUS);
    break;
  }
}

#endif
